const http = require('http');
const fs = require('fs');
const cookie = require('cookie');

// Note that it's totally fine to use sync functions in these scripts
// Since there's only ever one request at a time

const config = require('./config');

// Callback that ends the process *when* the response is sent
const end = () => process.exit(0);

function generate(entries, respond) {
	
	const ejs = require('ejs');
	const moment = require('moment');
	
	const html = ejs.render(
		fs.readFileSync('index.ejs', { encoding: 'utf-8'}),
		{ entries, moment },
		{
			strict: true,
			localsName: 'data',
		}
	);
	
	if (!fs.existsSync(config.out)) fs.mkdirSync(config.out);
	if (!fs.existsSync(config.out + '/moment.min.js')) fs.writeFileSync(
		config.out + '/moment.min.js',
		fs.readFileSync(require.resolve('moment/min/moment.min.js'))
	);
	
	fs.writeFileSync(config.out + '/index.html', html);
	
	respond({ success: true });
	
}

// Very simple RPC API
const api = {
	
	post(params, respond) {
	
		if (params.password !== config.password) return respond(403, 'Invalid password');
		if (!params.text.trim()) return respond(409, 'You forgot to enter anything :)');
		
		const sqlite3 = require('sqlite3');
		const db = new sqlite3.Database('hairsplitter.sqlite3');
		
		db.serialize(() => {
			
			db.run(`CREATE TABLE IF NOT EXISTS entries (
				id INTEGER PRIMARY KEY AUTOINCREMENT,
				time INTEGER,
				text TEXT
			)`);
			
			db.run('INSERT INTO entries (time, text) VALUES (?, ?)', [Date.now(), params.text]);
			
			db.all('SELECT * FROM entries ORDER BY id DESC', (err, rows) => {
				if (err) respond(500, 'Server error');
				generate(rows, respond);
			});
			
		});
		
	}
	
};

// Example request handling function
function handleRequest(req, res) {
	
	const split = req.url.split('?')[0].split('/');
	const args = split.slice(3);
	
	const cookies = cookie.parse(req.headers.cookie || '');
	
	if (!args[0]) {
		
		let csrf = cookies.csrf;
		if (!csrf) {
			csrf = require('crypto').randomBytes(16).toString('hex');
			res.setHeader('Set-Cookie', cookie.serialize('csrf', csrf, { path: '/' }));
		}
		
		res.end(require('ejs').render(
			fs.readFileSync('form.ejs', { encoding: 'utf-8'}),
			{
				csrf,
				prefix: split.slice(0, 3).join('/')
			},
			{
				strict: true,
				localsName: 'data',
			}
		), end);
		
	}
	else if (args[0] === 'api') {
		
		const fn = args[1];
		if (!api[fn]) {
			res.statusCode = 404;
			res.end('API endpoint not found', end);
			return;
		}
		
		const chunks = [];
		req.on('data', ch => chunks.push(ch));
		req.on('end', () => {
			
			const body = Buffer.concat(chunks).toString('utf8');
			let params;
			try {
				params = JSON.parse(body);
			}
			catch (e) {
				res.statusCode = 400;
				res.end(JSON.stringify({ error: 'Could not parse request' }), end);
				return;
			}
			
			if (!cookies.csrf || params.csrf !== cookies.csrf) {
				res.statusCode = 403;
				res.end('CSRF error', end);
				return;
			}
			
			api[fn](params, (response, emsg) => {
				if (typeof response === 'number') {
					res.statusCode = response;
					res.end(JSON.stringify({ error: emsg || ('Error' + code) }), end);
				}
				else {
					res.end(JSON.stringify(response), end);
				}
			});
			
		});
		
	}
	else {
		
		res.statusCode = 404;
		res.end('Not found', end);
		
	}
	
}

// Server initialization
// This is the 2038-specific bit

const server = http.createServer(handleRequest);

// Listen on the port passed via env
server.listen(parseInt(process.env._2038_PORT), e => {
	
	if (e) {
		console.error(e);
		process.exit(1);
	}
	
	// Signal to the parent process that we're ready to accept the request
	console.log(process.env._2038_READY);
	
});
